"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PM = void 0;
exports.getPackageManagerCommand = getPackageManagerCommand;
exports.detectPackageManager = detectPackageManager;
const path = require("path");
const fs = require("fs");
var PM;
(function (PM) {
    PM["NPM"] = "npm";
    PM["YARN"] = "yarn";
    PM["PNPM"] = "pnpm";
    PM["YARN_BERRY"] = "yarn-berry";
})(PM || (exports.PM = PM = {}));
function detectPackageManagerByEnv() {
    var _a, _b, _c, _d;
    if (process.env.npm_config_user_agent) {
        const userAgent = process.env.npm_config_user_agent;
        if (userAgent.includes("pnpm")) {
            return PM.PNPM;
        }
        if (userAgent.includes("yarn")) {
            if (userAgent.includes("yarn/")) {
                const version = userAgent.match(/yarn\/(\d+)\./);
                if (version && parseInt(version[1]) >= 2) {
                    return PM.YARN_BERRY;
                }
            }
            return PM.YARN;
        }
        if (userAgent.includes("npm")) {
            return PM.NPM;
        }
    }
    if (process.env.npm_execpath) {
        const execPath = process.env.npm_execpath.toLowerCase();
        if (execPath.includes("pnpm")) {
            return PM.PNPM;
        }
        if (execPath.includes("yarn")) {
            if (execPath.includes("berry") || ((_a = process.env.YARN_VERSION) === null || _a === void 0 ? void 0 : _a.startsWith("2.")) || ((_b = process.env.YARN_VERSION) === null || _b === void 0 ? void 0 : _b.startsWith("3."))) {
                return PM.YARN_BERRY;
            }
            return PM.YARN;
        }
        if (execPath.includes("npm")) {
            return PM.NPM;
        }
    }
    if (process.env.PNPM_HOME) {
        return PM.PNPM;
    }
    if (process.env.YARN_REGISTRY) {
        if (((_c = process.env.YARN_VERSION) === null || _c === void 0 ? void 0 : _c.startsWith("2.")) || ((_d = process.env.YARN_VERSION) === null || _d === void 0 ? void 0 : _d.startsWith("3."))) {
            return PM.YARN_BERRY;
        }
        return PM.YARN;
    }
    if (process.env.npm_package_json) {
        return PM.NPM;
    }
    // return default
    return PM.NPM;
}
function getPackageManagerCommand(pm) {
    let cmd = pm;
    if (pm === PM.YARN_BERRY || process.env.FORCE_YARN === "true") {
        cmd = PM.YARN;
    }
    return `${cmd}${process.platform === "win32" ? ".cmd" : ""}`;
}
function detectPackageManager(cwd) {
    const isYarnLockFileExists = fs.existsSync(path.join(cwd, "yarn.lock"));
    const isPnpmLockFileExists = fs.existsSync(path.join(cwd, "pnpm-lock.yaml"));
    const isNpmLockFileExists = fs.existsSync(path.join(cwd, "package-lock.json"));
    if (isYarnLockFileExists && !isPnpmLockFileExists && !isNpmLockFileExists) {
        // check if yarn is berry
        const pm = detectPackageManagerByEnv();
        if (pm === PM.YARN_BERRY) {
            return PM.YARN_BERRY;
        }
        return PM.YARN;
    }
    if (isPnpmLockFileExists && !isYarnLockFileExists && !isNpmLockFileExists) {
        return PM.PNPM;
    }
    if (isNpmLockFileExists && !isYarnLockFileExists && !isPnpmLockFileExists) {
        return PM.NPM;
    }
    // if there are no lock files or multiple lock files, return the package manager from env
    return detectPackageManagerByEnv();
}
//# sourceMappingURL=packageManager.js.map