"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PnpmNodeModulesCollector = void 0;
const lazy_val_1 = require("lazy-val");
const nodeModulesCollector_1 = require("./nodeModulesCollector");
const builder_util_1 = require("builder-util");
const path = require("path");
const fs = require("fs");
class PnpmNodeModulesCollector extends nodeModulesCollector_1.NodeModulesCollector {
    constructor(rootDir) {
        super(rootDir);
        this.pmCommand = _a.pmCommand;
        this.installOptions = this.pmCommand.value.then(cmd => ({ cmd, args: ["install", "--frozen-lockfile"], lockfile: "pnpm-lock.yaml" }));
    }
    getArgs() {
        return ["list", "--prod", "--json", "--depth", "Infinity"];
    }
    extractProductionDependencyGraph(tree, dependencyId) {
        if (this.productionGraph[dependencyId]) {
            return;
        }
        const p = path.normalize(this.resolvePath(tree.path));
        const packageJson = require(path.join(p, "package.json"));
        const prodDependencies = { ...packageJson.dependencies, ...packageJson.optionalDependencies };
        const deps = { ...(tree.dependencies || {}), ...(tree.optionalDependencies || {}) };
        this.productionGraph[dependencyId] = { dependencies: [] };
        const dependencies = Object.entries(deps)
            .filter(([packageName, dependency]) => {
            // First check if it's in production dependencies
            if (!prodDependencies[packageName]) {
                return false;
            }
            // Then check if optional dependency path exists
            if (packageJson.optionalDependencies && packageJson.optionalDependencies[packageName] && !fs.existsSync(dependency.path)) {
                builder_util_1.log.debug(null, `Optional dependency ${packageName}@${dependency.version} path doesn't exist: ${dependency.path}`);
                return false;
            }
            return true;
        })
            .map(([packageName, dependency]) => {
            const childDependencyId = `${packageName}@${dependency.version}`;
            this.extractProductionDependencyGraph(dependency, childDependencyId);
            return childDependencyId;
        });
        this.productionGraph[dependencyId] = { dependencies };
    }
    collectAllDependencies(tree) {
        // Collect regular dependencies
        for (const [key, value] of Object.entries(tree.dependencies || {})) {
            this.allDependencies.set(`${key}@${value.version}`, value);
            this.collectAllDependencies(value);
        }
        // Collect optional dependencies if they exist
        for (const [key, value] of Object.entries(tree.optionalDependencies || {})) {
            this.allDependencies.set(`${key}@${value.version}`, value);
            this.collectAllDependencies(value);
        }
    }
    parseDependenciesTree(jsonBlob) {
        const dependencyTree = JSON.parse(jsonBlob);
        // pnpm returns an array of dependency trees
        return dependencyTree[0];
    }
}
exports.PnpmNodeModulesCollector = PnpmNodeModulesCollector;
_a = PnpmNodeModulesCollector;
PnpmNodeModulesCollector.pmCommand = new lazy_val_1.Lazy(async () => {
    if (process.platform === "win32") {
        try {
            await (0, builder_util_1.exec)("pnpm", ["--version"]);
        }
        catch (_error) {
            builder_util_1.log.debug(null, "pnpm not detected, falling back to pnpm.cmd");
            return "pnpm.cmd";
        }
    }
    return "pnpm";
});
//# sourceMappingURL=pnpmNodeModulesCollector.js.map